# Generated by roxygen2: do not edit by hand

S3method("+",CFClimatology)
S3method("+",CFTime)
S3method("==",CFTime)
S3method("[",CFTime)
S3method(as.character,CFTime)
S3method(cut,CFTime)
S3method(length,CFTime)
S3method(range,CFTime)
S3method(str,CFTime)
export("bounds<-")
export(CFClimatology)
export(CFTime)
export(CFfactor)
export(CFfactor_coverage)
export(CFfactor_units)
export(CFtime)
export(as_timestamp)
export(bounds)
export(calendar)
export(definition)
export(indexOf)
export(is_complete)
export(month_days)
export(offsets)
export(origin)
export(parse_timestamps)
export(resolution)
export(slice)
export(timezone)
export(unit)
import(R6)
importFrom(stats,na.omit)
