% Generated by roxygen2: do not edit by hand % Please edit documentation in R/CFClimatology.R \docType{class} \name{CFClimatology} \alias{CFClimatology} \title{CFClimatology class} \description{ This class represents a climatological time coordinate, both its "calendar" and its "climatology" bounds values. The "calendar" portion is managed by the \link{CFTime} class, from which this class is inherited. This class implements the specific behaviour of the climatological bounds values and includes methods to query the structure of the climatological data. } \references{ https://cf-convention.github.io/Data/cf-conventions/cf-conventions-1.13/cf-conventions.html#climatological-statistics } \section{Super class}{ \code{\link[CFtime:CFTime]{CFtime::CFTime}} -> \code{CFClimatology} } \section{Active bindings}{ \if{html}{\out{
}} \describe{ \item{\code{period}}{(read-only) Character string indicating the period during the year over which the climatology was calculated.} \item{\code{years}}{(read-only) Vector of two integer values indicating the years over which the climatology was calculated.} \item{\code{friendlyClassName}}{Character string with class name for display purposes.} } \if{html}{\out{
}} } \section{Methods}{ \subsection{Public methods}{ \itemize{ \item \href{#method-CFClimatology-new}{\code{CFClimatology$new()}} \item \href{#method-CFClimatology-print}{\code{CFClimatology$print()}} \item \href{#method-CFClimatology-clone}{\code{CFClimatology$clone()}} } } \if{html}{\out{
Inherited methods

- `CFtime::CFTime$POSIX_compatible()`
- `CFtime::CFTime$as_timestamp()`
- `CFtime::CFTime$copy()`
- `CFtime::CFTime$cut()`
- `CFtime::CFTime$doy()`
- `CFtime::CFTime$equidistant()`
- `CFtime::CFTime$factor()`
- `CFtime::CFTime$factor_coverage()`
- `CFtime::CFTime$factor_units()`
- `CFtime::CFTime$format()`
- `CFtime::CFTime$get_bounds()`
- `CFtime::CFTime$indexOf()`
- `CFtime::CFTime$range()`
- `CFtime::CFTime$set_bounds()`
- `CFtime::CFTime$slice()`
- `CFtime::CFTime$subset()`

}} \if{html}{\out{

}} \if{html}{\out{}} \if{latex}{\out{\hypertarget{method-CFClimatology-new}{}}} \subsection{Method \code{new()}}{ Create a new instance of this class. \subsection{Usage}{ \if{html}{\out{
}}\preformatted{CFClimatology$new(definition, calendar = "standard", offsets, bounds)}\if{html}{\out{
}} } \subsection{Arguments}{ \if{html}{\out{
}} \describe{ \item{\code{definition}}{Character string of the units and origin of the calendar.} \item{\code{calendar}}{Character string of the calendar to use. Must be one of the values permitted by the CF Metadata Conventions. If \code{NULL}, the "standard" calendar will be used.} \item{\code{offsets}}{Numeric or character vector. When numeric, a vector of offsets from the origin in the time series. When a character vector of length 2 or more, timestamps in ISO8601 or UDUNITS format.} \item{\code{bounds}}{The climatological bounds for the offsets. A \code{matrix} with columns for \code{offsets} and low values in the first row, high values in the second row. The bounds will oftentimes overlap or be discontinuous.} } \if{html}{\out{
}} } } \if{html}{\out{

}} \if{html}{\out{}} \if{latex}{\out{\hypertarget{method-CFClimatology-print}{}}} \subsection{Method \code{print()}}{ Print a summary of the \code{CFClimatology} object to the console. \subsection{Usage}{ \if{html}{\out{
}}\preformatted{CFClimatology$print(...)}\if{html}{\out{
}} } \subsection{Arguments}{ \if{html}{\out{
}} \describe{ \item{\code{...}}{Ignored.} } \if{html}{\out{
}} } \subsection{Returns}{ \code{self} invisibly. } } \if{html}{\out{

}} \if{html}{\out{}} \if{latex}{\out{\hypertarget{method-CFClimatology-clone}{}}} \subsection{Method \code{clone()}}{ The objects of this class are cloneable with this method. \subsection{Usage}{ \if{html}{\out{
}}\preformatted{CFClimatology$clone(deep = FALSE)}\if{html}{\out{
}} } \subsection{Arguments}{ \if{html}{\out{
}} \describe{ \item{\code{deep}}{Whether to make a deep clone.} } \if{html}{\out{
}} } } }