% Generated by roxygen2: do not edit by hand % Please edit documentation in R/CFCalendarTAI.R \docType{class} \name{CFCalendarTAI} \alias{CFCalendarTAI} \title{International Atomic Time CF calendar} \description{ This class represents a calendar based on the International Atomic Time. Validity is from 1958 onwards, with dates represented using the Gregorian calendar. Given that this "calendar" is based on a universal clock, the concepts of leap second, time zone and daylight savings time do not apply. } \section{Super classes}{ \code{\link[CFtime:CFCalendar]{CFtime::CFCalendar}} -> \code{\link[CFtime:CFCalendarProleptic]{CFtime::CFCalendarProleptic}} -> \code{CFCalendarTAI} } \section{Methods}{ \subsection{Public methods}{ \itemize{ \item \href{#method-CFCalendarTAI-valid_days}{\code{CFCalendarTAI$valid_days()}} \item \href{#method-CFCalendarTAI-clone}{\code{CFCalendarTAI$clone()}} } } \if{html}{\out{
Inherited methods

- `CFtime::CFCalendar$add_day()`
- `CFtime::CFCalendar$doy()`
- `CFtime::CFCalendar$is_compatible()`
- `CFtime::CFCalendar$is_equivalent()`
- `CFtime::CFCalendar$offsets2time()`
- `CFtime::CFCalendar$parse()`
- `CFtime::CFCalendar$print()`
- `CFtime::CFCalendarProleptic$POSIX_compatible()`
- `CFtime::CFCalendarProleptic$date2offset()`
- `CFtime::CFCalendarProleptic$initialize()`
- `CFtime::CFCalendarProleptic$leap_year()`
- `CFtime::CFCalendarProleptic$month_days()`
- `CFtime::CFCalendarProleptic$offset2date()`

}} \if{html}{\out{

}} \if{html}{\out{}} \if{latex}{\out{\hypertarget{method-CFCalendarTAI-valid_days}{}}} \subsection{Method \code{valid_days()}}{ Indicate which of the supplied dates are valid. \subsection{Usage}{ \if{html}{\out{
}}\preformatted{CFCalendarTAI$valid_days(ymd)}\if{html}{\out{
}} } \subsection{Arguments}{ \if{html}{\out{
}} \describe{ \item{\code{ymd}}{\code{data.frame} with dates parsed into their parts in columns \code{year}, \code{month} and \code{day}. If present, the \code{tz} column is checked for illegal time zone offsets. Any other columns are disregarded.} } \if{html}{\out{
}} } \subsection{Returns}{ Logical vector with the same length as argument \code{ymd} has rows with \code{TRUE} for valid days and \code{FALSE} for invalid days, or \code{NA} where the row in argument \code{ymd} has \code{NA} values. } } \if{html}{\out{

}} \if{html}{\out{}} \if{latex}{\out{\hypertarget{method-CFCalendarTAI-clone}{}}} \subsection{Method \code{clone()}}{ The objects of this class are cloneable with this method. \subsection{Usage}{ \if{html}{\out{
}}\preformatted{CFCalendarTAI$clone(deep = FALSE)}\if{html}{\out{
}} } \subsection{Arguments}{ \if{html}{\out{
}} \describe{ \item{\code{deep}}{Whether to make a deep clone.} } \if{html}{\out{
}} } } }